require 'open3'
require_relative 'exceptions/tika_error'

module TikaMasala
  class FileNotFound < StandardError; end

  class Parser
    attr_reader :jar_path

    def initialize(jar_path = nil)
      if jar_path.nil?
        jar_path = File.expand_path(File.join('..', '..', '..', 'dependencies', 'tika-app-1.5.jar'), __FILE__)
      end

      raise FileNotFound, "Jar #{jar_path} does not exist" unless File.exists?(jar_path)

      @jar_path = jar_path
    end

    def parse(file)
      java_exec('--text', file)
    end

    def metadata(file)
      java_exec('--metadata', file)
    end

    def detect_type(file)
      java_exec('--detect', file)
    end

    def java_exec(*args)
      stdin, stdout, stderr, wait_thr = Open3.popen3("java -jar #{@jar_path.shellescape} #{args.shelljoin}")

      exitstatus = wait_thr.value

      if exitstatus != 0
        expection = TikaError.new(
          stdout: stdout.read,
          stderr: stderr.read,
          exitstatus: exitstatus
        )
        raise expection
      else
        stdout.read
      end
    ensure
      stdin.close
      stdout.close
      stderr.close
    end
  end
end
